set packageName tcf;

package provide $packageName 1.0;

set ${packageName}_version								1.0;
set env([string toupper $packageName]_LIBRARY)	[file dirname [info script]];

if {[lsearch $auto_path $env([string toupper $packageName]_LIBRARY)] < 0} {
	lappend auto_path $env([string toupper $packageName]_LIBRARY);
}

namespace eval $packageName {
	global packageName;
	variable this;
	variable parent;

	variable EXCEPTIONS;
	variable EXCEPTIONSCFG;
	variable EXCEPTIONSMSG;
	variable EXCEPTIONSCODE;
}

proc $packageName::$packageName {} {
	variable this;
	variable parent;

	variable EXCEPTIONS;
	variable EXCEPTIONSCFG;
	variable EXCEPTIONSMSG;
	variable EXCEPTIONSCODE;

	set this		[namespace current];
	set parent	[namespace parent $this];

	set EXCEPTIONSCFG(on)				1;
	set EXCEPTIONSCFG(lastException)	"";
	set EXCEPTIONSCFG(lastMessage)	"";
	set EXCEPTIONSCFG(lastCode)		{};

	namespace export exceptions {[a-z]*};
}

proc $packageName::~$packageName {} {
	variable this;

	namespace delete $this;
}

proc $packageName::class {} {
	variable this;

	return [namespace tail $this];
};

proc $packageName::classes { { pattern * } } {
	variable this;

	return [namespace children ${this} $pattern];
};

proc $packageName::vars { { pattern * } } {
	variable this;

	return [info vars ${this}::$pattern];
};

proc $packageName::methods { { pattern * } } {
	variable this;

	return [info commands ${this}::$pattern];
};

proc $packageName::exceptions { args } {
	variable this;

	if {[llength $args] < 1} {
		return -code error "wrong # args: should be \"exceptions option arg ?arg ...?\"";
	};

	set option	[lindex $args 0];
	set args		[lrange $args 1 end];
	set Option	[string toupper [string index $option 0]][string tolower [string range $option 1 end]];
	set proc		${this}::$Option;

	if { [llength [info commands ${proc}]] == 1 } {
		return [eval $proc $args];
	}

	if { [llength [set procs [info commands ${proc}*]]] > 0 } {
		regsub -nocase -all -- ${this}:: [string tolower [lsort $procs]] {} ambiguous;

		return -code error "ambiguous option name \"$option\": $ambiguous";
	}

	regsub -nocase -all -- ${this}:: [string tolower [lsort [info commands ${this}::\[A-Z\]*]]] {} options;
	regsub -nocase -all -- (~?[${this}::class])|methods|class|classes $options {} options;

	set options	"[string trim [join [lrange $options 0 [expr [llength $options]-1]] ", "] " ,"] or [lindex $options end]";

	return -code error "bad option \"$option\": must be $options";
};

proc $packageName::Read {args} {
	variable this;
	variable EXCEPTIONSCFG;

	set argc 0;
	foreach arg $args {
		set argv($argc) $arg;
		incr argc;
	};

	if {$argc} {
		switch -glob -- $argv(0) {
			-file	{
				if { $argc == 2 } {
					set file $argv(1);

					if { [file exists $file] } {
						if { [file isfile $file] } {
							if { [file readable $file] } {
								if { ![catch { open $file r } id] } {
									set data [read -nonewline $id];
									close $id;

									${this}::Read -string $data;

									return;
								} else {
									return -code error "couldn't open file \"$file\": unknown problem (exceptions read)";
								};
							} else {
								return -code error "couldn't open file \"$file\": file is not readable (exceptions read)";
							};
						} else {
							return -code error "couldn't open file \"$file\": file is invalid (exceptions read)";
						};
					} else {
						return -code error "couldn't open file \"$file\": no such file (exceptions read)";
					};
				} else {
					return -code error "wrong # args: should be \"exceptions read -file filename\"";
				};
			}
			-list	{
				if { $argc == 2 } {
					set list $argv(1);

					if { [llength $list] % 4 == 0 } {
						foreach { id msg code } $list {
							if { $code == {} } {
								set code NONE
							}
							${this}::Add $id $msg $code
						}

						return
					} else {
						return -code error "couldn't use list: list must have a 4*n number of elements (exceptions read)"
					}
				} else {
					return -code error "wrong # args: should be \"exceptions read -list listdata\""
				}
			}
			-string	{
				if { $argc == 2 } {
					set string $argv(1)

					set list {}
					foreach line [split $string \n] {
						if { ( $line == {} ) || [regexp -- ^# $line] } { continue }
						foreach { id msg code } $line {}
						if { $code == {} } {
							set code NONE
						}
						${this}::Add $id $msg $code
					}

					return
				} else {
					return -code error "wrong # args: should be \"exceptions read -string stringdata\""
				}
			}
			-*		{
				return -code error "bad option \"$argv(0)\": must be -file, -list or -string"
			}
			default	{
				if { $argc == 1 } {
					${this}::Read -file $argv(0)
				} else {
					return -code error "wrong # args: should be \"exceptions read filename\""
				}
			}
		}
	} else {
		return -code error "wrong # args: should be \"exceptions read ?option? source\""
	}
}

proc $packageName::On { } {
	variable EXCEPTIONSCFG

	set EXCEPTIONSCFG(on) 1
}

proc $packageName::Off { } {
	variable EXCEPTIONSCFG

	set EXCEPTIONSCFG(on) 0
}

proc $packageName::Ison { } {
	variable EXCEPTIONSCFG

	return $EXCEPTIONSCFG(on)
}

proc $packageName::Isoff { } {
	variable EXCEPTIONSCFG

	return !$EXCEPTIONSCFG(on)
}

proc $packageName::Israised { args } {
	if { ( [llength $args] == 0 ) || ( [llength $args] == 1 ) } {
		variable EXCEPTIONSCFG

		set exception	[lindex $args 0]

		if { $exception == {} } {
			return $EXCEPTIONSCFG(lastException)
		} else {
			if { $EXCEPTIONSCFG(lastException) == $exception } {
				return 1
			} else {
				return 0
			}
		}
	} else {
		return -code error "wrong # args: should be \"exception israised ?exception?\""
	}
}

proc $packageName::Lastraised { } {
	variable EXCEPTIONSCFG
	return [list $EXCEPTIONSCFG(lastException) $EXCEPTIONSCFG(lastMessage) $EXCEPTIONSCFG(lastCode)]
}

proc $packageName::Current { } {
	variable this

	return [${this}::Lastraised]
}

proc $packageName::Raise { args } {
	variable this

	if { ( [llength $args] >= 1 ) && ( [llength $args] <= 3 ) } {
		variable EXCEPTIONSCFG
		variable EXCEPTIONSMSG
		variable EXCEPTIONSCODE

		set exception	[lindex $args 0]
		set args			[lrange $args 1 end]

		if { [${this}::Ison] } {
			if { ![${this}::Exists $exception] } {
				eval ${this}::Add $exception $args
			}

			if { [${this}::Iscatchable $exception] } {
				set EXCEPTIONSCFG(lastException)	$exception
				set EXCEPTIONSCFG(lastMessage)	$EXCEPTIONSMSG($exception)
				set EXCEPTIONSCFG(lastCode)		$EXCEPTIONSCODE($exception)

				error $EXCEPTIONSMSG($exception) "" $EXCEPTIONSCODE($exception)
			}
		}
	} else {
		return -code error "wrong # args: should be \"exceptions raise exception ?exceptionMsg exceptionCode?\""
	}
}

proc $packageName::Catchnraise { args } {
	variable this

	set argc 0
	foreach arg $args {
		set argv($argc) $arg
		incr argc
	}

	if { $argc } {
		variable EXCEPTIONSCFG
		variable EXCEPTIONSMSG
		variable EXCEPTIONSCODE

		# collecting the arguements

		switch -- $argv(0) {
			-global {
				if { $argc < 3 } {
					return -code error "wrong # args: should be \"exceptions catchnraise -global {} exception ?exceptionMsg? ?exceptionCode?\""
				} else {
					set uplevel			#0
					set block			$argv(1)
					set exception		$argv(2)
					set exceptionDef	{}
				} 
				
				if { $argc == 4 } {
					set exceptionDef	[lrange $args 3 end]
				}
			}
			-level  {
				if { $argc < 4 } {
					return -code error "wrong # args: should be \"exceptions catchnraise -level levelnumber {} exception ?exceptionMsg? ?exceptionCode?\""
				} else {
					set uplevel			$argv(1)
					set block			$argv(2)
					set exception		$argv(3)
					set exceptionInfo	{}
				}

				if { $argc == 5 } {
					set exceptionInfo	[lrange $args 4 end]
				}
			}
			default {
				if { $argc < 2 } {
					return -code error "wrong # args: should be \"exceptions catchnraise {} exception ?exceptionMsg? ?exceptionCode?\""
				} else {
					set uplevel			1
					set block			$argv(0)
					set exception		$argv(1)
					set exceptionInfo	{}
				}

				if { $argc == 3 } {
					set exceptionInfo	[lrange $args 2 end]
				}
			}
		}

		# verifying the arguements

		if { [regexp -- {^(#?)([0-9]+)} $uplevel whole absolute level] } {
			if { $level + 1 <= [info level] } {
				if { $absolute == {} } {
					incr level
				}
				set uplevel $absolute$level
			} else {
				return -code error "couldn't use level \"$level\": must be \"0 <= level <= [info level]\" (exceptions catchnraise)"
			}
		} else {
			return -code error "couldn't use level \"$uplevel\": must be \"?#?levelnumber\" (exceptions catchnraise)"
		}

		if { ![info complete $block] } {
			return -code error "couldn't use executional block: missing closing brace, bracket or \" (exceptions catchnraise)"
		}

		if { ![${this}::Exists $exception] } {
			eval ${this}::Add $exception $exceptionInfo
		}

		# executing the execution block

		if { [catch { uplevel $uplevel $block } result] } {
			global errorInfo errorCode

			set reason	$errorInfo
			set code		$errorCode

			if { [${this}::Iscatchable $exception] } {
				if { $EXCEPTIONSMSG($exception) != "" } {
					set result $EXCEPTIONSMSG($exception)
				}
				set code	$EXCEPTIONSCODE($exception)
				
				set EXCEPTIONSCFG(lastException)	$exception
				set EXCEPTIONSCFG(lastMessage)	$result
				set EXCEPTIONSCFG(lastCode)		$code
			}

			error $result $reason $code
		} else {
			return $result
		}
	} else {
		return -code error "wrong # args: should be \"exceptions catchnraise ?option? {..} exception ?exceptionMsg? ?exceptionCode?\""
	}
}

proc $packageName::Lower { } {
	variable EXCEPTIONSCFG

	set EXCEPTIONSCFG(lastException)	""
	set EXCEPTIONSCFG(lastMessage)	""
	set EXCEPTIONSCFG(lastCode)		""
}

proc $packageName::Enable { args } {
	variable this

	variable EXCEPTIONS

	if { [llength $args] > 0 } {
		foreach exception $args {
			if { [${this}::Exists $exception] } {
				set EXCEPTIONS($exception) 1
			} else {
				return -code error "couldn't use exception \"$exception\": exception not found (exceptions enable)"
			}
		}
	} else {
		return -code error "wrong # args: should be \"exceptions enable exception ?exception ...?\""
	}
}

proc $packageName::Disable { args } {
	variable this

	variable EXCEPTIONS

	if { [llength $args] > 0 } {
		foreach exception $args {
			if { [${this}::Exists $exception] } {
				set EXCEPTIONS($exception) 0
			} else {
				return -code error "couldn't use exception \"$exception\": exception not found (exceptions disable)"
			}
		}
	} else {
		return -code error "wrong # args: should be \"exceptions disable exception ?exception ...?\""
	}
}

proc $packageName::Iscatchable { args } {
	variable this

	if { [llength $args] == 1 } {
		variable EXCEPTIONS

		set exception [lindex $args 0]

		if { [${this}::Exists $exception] } {
			return $EXCEPTIONS($exception)
		} else {
			return -code error "couldn't use exception \"$exception\": exception not found (exceptions iscatchable)"
		}
	} else {
		return -code error "wrong # args: should be \"exceptions iscatchable exception\""
	}
}

proc $packageName::Exists { args } {
	variable this

	if { [llength $args] == 1 } {
		set exception [lindex $args 0]

		if { [lsearch -glob [${this}::Info] $exception] >= 0 } {
			return 1
		} else {
			return 0
		}
	} else {
		return -code error "wrong # args: should be \"exceptions exists exception\""
	}
}

proc $packageName::Exceptioninfo { args } {
	variable this

	if { [llength $args] == 1 } {
		variable EXCEPTIONSMSG
		variable EXCEPTIONSCODE

		set exception [lindex $args 0]

		if { [${this}::Exists $exception] } {
			return [list $EXCEPTIONSMSG($exception) $EXCEPTIONSCODE($exception)]
		} else {
			return -code error "couldn't use exception \"$exception\": exception not found (exceptions exceptioninfo)"
		}
	} else {
		return -code error "wrong # args: should be \"exceptions exceptioninfo exception\""
	}
}

proc $packageName::Info { args } {
	variable EXCEPTIONS

	switch -- [llength $args] {
		0   { set result [array names EXCEPTIONS] }
		1   { set result [array names $EXCEPTIONS [lindex $args 0]] }
		default { 
			return -code error "wrong # args: should be \"exceptions info ?pattern?\""
		}
	}

	return $result
}

proc $packageName::Def { args } {
	variable this

	if { ( [llength $args] >= 1 ) && ( [llength $args] <= 3 ) } {
		return [eval ${this}::Set $args]
	} else {
		return -code error "wrong # args: should be \"exceptions def exception ?exceptionMsg? ?exceptionCode?\""
	}
}

proc $packageName::Undef { args } {
	variable this

	if { [llength $args] >= 1 } {
		eval ${this}::Unset $args
	} else {
		return -code error "wrong # args: should be \"exceptions undef exception ?exception ...?\""
	}
}

proc $packageName::Set { args } {
	variable this

	if { ( [llength $args] >= 1 ) && ( [llength $args] <= 3 ) } {
		if { ![${this}::Exists [lindex $args 0]] } {
			return [eval ${this}::Add $args]
		} else {
			return [eval ${this}::Change $args]
		}
	} else {
		return -code error "wrong # args: should be \"exceptions set exception ?exceptionMsg? ?exceptionCode?\""
	}

	return
}

proc $packageName::Unset { args } {
	variable this

	if { [llength $args] >= 1 } {
		eval ${this}::Delete $args
	} else {
		return -code error "wrong # args: should be \"exceptions unset exception ?exception ...?\""
	}
}

proc $packageName::Add { args } {
	variable this

	variable EXCEPTIONS
	variable EXCEPTIONSMSG
	variable EXCEPTIONSCODE

	set argc 0
	foreach arg $args {
		set argv($argc) $arg
		incr argc
	}

	if { $argc } {
		set exception	$argv(0)
		set message		""
		set code			$exception

		if { $argc >= 2 } {
			set message $argv(1)
		}

		if { $argc == 3 } {
			set code	$argv(2)
		} elseif { $argc > 3 } {
			return -code error "wrong # args: should be \"exceptions add exception ?exceptionMsg? ?exceptionCode?\""
		}

		if { ![${this}::Exists $exception] } {
			set EXCEPTIONS($exception)			1
			set EXCEPTIONSMSG($exception)		$message
			set EXCEPTIONSCODE($exception)	$code

			return $exception
		} else {
			return -code error "couldn't use exception \"$exception\": exception already exists (exceptions add)"
		}
	} else {
		return -code error "wrong # args: should be \"exceptions add exception ?exceptionMsg? ?exceptionCode?\""
	}
}

proc $packageName::Del { args } {
	variable this

	variable EXCEPTIONS
	variable EXCEPTIONSMSG
	variable EXCEPTIONSCODE

	if { [llength $args] > 0 } {
		if { $args == "all" } {
			set args [${this}::Info]
		}

		foreach exception $args {
			if { [${this}::Exists $exception] } {
				unset EXCEPTIONS($exception)
				unset EXCEPTIONSMSG($exception)
				unset EXCEPTIONSCODE($exception)

				if { [${this}::Israised $exception] } {
					set EXCEPTIONSCFG(lastException)	""
					set EXCEPTIONSCFG(lastMessage)	""
					set EXCEPTIONSCFG(lastCode)		""
				}
			} else {
				return -code error "couldn't use exception \"$exception\": exception not found (exceptions del)"
			}
		}
	} else {
		return -code error "wrong # args: should be \"exceptions del exception ?exception?\""
	}

	return
}

proc $packageName::Clear { } {
	variable this

	${this}::Del all
}

proc $packageName::Change { args } {
	variable this

	set argc 0
	foreach arg $args {
		set argv($argc) $arg
		incr argc
	}
	
	if { ( $argc >= 2 ) && ( $argc <= 3 ) } {
		set exception	$argv(0)
		if { [${this}::Exists $exception] } {
			if { $argc >= 2 } {
				return [${this}::Changemessage $exception $argv(1)]
			}
			
			if { $argc == 3 } {
				return [${this}::Changecode $exception $argv(2)]
			}
		} else {
			return -code error "couldn't use exception \"$exception\": exception not found (exceptions change)"
		}
	} else {
		return -code error "wrong # args: should be \"exceptions change exception exceptionMsg ?exceptionCode?\""
	}
}

proc $packageName::Changemessage { args } {
	variable this

	if { ( [llength $args] >= 1 ) && ( [llength $args] <= 2 ) } {
		variable EXCEPTIONSMSG

		set exception		[lindex $args 0]
		set exceptionMsg	[lindex $args 1]

		if { [${this}::Exists $exception] } {
			set EXCEPTIONSMSG($exception) $exceptionMsg
		} else {
			return -code error "couldn't use exception \"$exception\": exception not found (exceptions changemessage)"
		}

		return $exception
	} else {
		return -code error "wrong # args: should be \"exceptions changemessage exception ?exceptionMsg?\""
	}
}

proc $packageName::Changecode { args } {
	variable this

	if { ( [llength $args] >= 1 ) && ( [llength $args] <= 2 ) } {
		variable EXCEPTIONSCODE

		set exception		[lindex $args 0]
		set exceptionCode	[lindex $args 1]

		if { [${this}::Exists $exception] } {
			if { $exceptionCode == {} } {
				set exceptionCode $exception
			}

			set EXCEPTIONSCODE($exception) $exceptionCode
		} else {
			return -code error "couldn't use exception \"$exception\": exception not found (exceptions changecode)"
		}

		return $exception
	} else {
		return -code error "wrong # args: should be \"exceptions changecode exception ?exceptionMsg?\""
	}
}

proc $packageName::Reset { } {
	variable this

	variable EXCEPTIONSCFG

	${this}::Del all

	set EXCEPTIONSCFG(lastException)	""
	set EXCEPTIONSCFG(lastMessage)	""
	set EXCEPTIONSCFG(lastCode)		""
}

proc $packageName::try { args } {
	variable this

	set argc	0
	foreach arg $args {
		set argv($argc) $arg
		incr argc
	}

	if { $argc } {
		# collecting an organizing the arguments

		set tryArray try[clock clicks]

		set ${tryArray}(catch:exceptions) {}
		switch -- $argv(0) {
			-global {
				if { ( [lsearch {2 4 5} $argc] >= 0 ) || ( ( $argc - 4 ) % 3 == 0 ) } {
					set uplevel							#0
					set ${tryArray}(common:block)	$argv(1)

					if { ( $argc == 5 ) && ( $argv(2) == "catch"  ) } {
						set ${tryArray}(common:catch)   1
						
						set exception								$argv(3)
						set ${tryArray}(catch:$exception)	$argv(4)

						lappend ${tryArray}(catch:exceptions) $exception
					} elseif { ( $argc > 5 ) && ( ( $argc - 4 ) % 3 == 0 ) } {
						set ${tryArray}(common:catch)	1

						set catchNo	1
						foreach { catch exception block } [lrange $args 2 end] {
							if { $block != {} } {
								if { $catch == "catch" } {
									lappend ${tryArray}(catch:exceptions) $exception

									set ${tryArray}(catch:$exception)	$block
								} else {
									return -code error "couldn't build catch-block number $catchNo: missing keyword catch"
								}
							}

							incr catchNo
						}
					} elseif { $argc < 5 } {
						set ${tryArray}(common:catch)	0
					} else {
						return -code error "couldn't build catch-block: invalid structure or # of args"
					}

					if { ( $argc - 2 ) % 3 == 2 } {
						set ${tryArray}(common:finally)	1
						set ${tryArray}(finally:block)	[lindex $args end]
					} else {
						set ${tryArray}(common:finally)	0
					}
				} else {
					return -code error "wrong # args, must be: \"try -global {..} ?catch exception {..} ...? ?finally {..}?\""
				}
			}
			-level  {
				if { ( $argc % 3 == 0 ) || ( ( $argc - 2 ) % 3 == 0 ) } {
					set uplevel							$argv(1)
					set ${tryArray}(common:block)	$argv(2)

					if { ( $argc == 6 ) && ( $argv(3) == "catch"  ) } {
						set ${tryArray}(common:catch)	1
						
						set exception								$argv(4)
						set ${tryArray}(catch:$exception)	$argv(5)

						lappend ${tryArray}(catch:exceptions) $exception
					} elseif { ( $argc > 6 ) && ( ( $argc - 5 ) % 3 == 0 ) } {
						set ${tryArray}(common:catch)	1

						set catchNo	1
						foreach { catch exception block } [lrange $args 3 end] {
							if { $block != {} } {
								if { $catch == "catch" } {
									lappend ${tryArray}(catch:exceptions) $exception

									set ${tryArray}(catch:$exception)	$block
								} else {
									return -code error "couldn't build catch-block number $catchNo: missing keyword catch"
								}
							}

							incr catchNo
						}
					} elseif { $argc < 6 } {
						set ${tryArray}(common:catch)	0
					} else {
						return -code error "couldn't build catch-block: invalid structure or # of args"
					}

					if { $argc % 3 == 2 } {
						set ${tryArray}(common:finally)	1
						set ${tryArray}(finally:block)	[lindex $args end]
					} else {
						set ${tryArray}(common:finally)	0
					}
				} else {
					return -code error "wrong # args: should be \"try -level levelnumber {..} ?catch exception {..} ...? ?finally {..}?\""
				}
			}
			default {
				if { ( [lsearch {1 3 4} $argc] >= 0 ) || 
					 ( ( $argc - 1 ) % 3 == 0 ) || ( $argc % 3 == 0 ) } {
					set uplevel							1
					set ${tryArray}(common:block)	$argv(0)

					if { ( $argc == 4 ) && ( $argv(1) == "catch"  ) } {
						set ${tryArray}(common:catch)	1
						
						set exception								$argv(2)
						set ${tryArray}(catch:$exception)	$argv(3)

						lappend ${tryArray}(catch:exceptions) $exception
					} elseif { ( $argc > 4 ) && ( ( $argc - 3 ) % 3 == 0 ) } {
						set ${tryArray}(common:catch)	1

						set catchNo	1
						foreach { catch exception block } [lrange $args 1 end] {
							if { $block != {} } {
								if { $catch == "catch" } {
									lappend ${tryArray}(catch:exceptions) $exception

									set ${tryArray}(catch:$exception)	$block
								} else {
									return -code error "couldn't build catch-block number $catchNo: missing keyword catch"
								}
							}

							incr catchNo
						}
					} elseif { $argc < 4 } {
						set ${tryArray}(common:catch)	0
					} else {
						return -code error "couldn't build catch-block: invalid structure or # of args"
					}

					if { $argc % 3 == 0 } {
						set ${tryArray}(common:finally)	1
						set ${tryArray}(finally:block)	[lindex $args end]
					} else {
						set ${tryArray}(common:finally)	0
					}
				} else {
					return -code error "wrong # args: should be \"try {..} ?catch exception {..} ...? ?finally {..}?\""
				}
			}
		}
			
		# verifying  the arguments
		
		# verifying the level for the uplevel command

		if { [regexp -- {^(#?)([0-9]+)} $uplevel whole absolute level] } {
			if { $level <= [info level] } {
				if { [regexp -- level $argv(0)] && ( $absolute == {} ) } {
					incr level
				}
				set ${tryArray}(common:uplevel) $absolute$level
			} else {
				unset ${tryArray}
							
				return -code error "couln't use level \"$level\": must be \"0 <= level <= [info level]\" (try)"
			}
		} else {
			unset ${tryArray}
							
			return -code error "couldn't use level \"$uplevel\": must be \"?#?levelnumber\" (try)"
		}

		# verifying the "completeness" of the execution blocks

		if { ![info complete [set ${tryArray}(common:block)]] } {
			unset ${tryArray}
						
			return -code error "couldn't use try block: missing closing brace, bracket or \" (try)"
		}
		foreach blockElem [concat [array names ${tryArray} catch:*]] {
			if { ![info complete [set ${tryArray}($blockElem)]] } {
				set exception [lindex [split $blockElem :] 1]

				unset ${tryArray}
							
				return -code error "couldn't use catch block for the exception \"$exception\": missing closing brace, bracket or \" (try)"
			}
		}
		if { [info exists ${tryArray}(finally:block)] && 
			 ![info complete [set ${tryArray}(finally:block)]] } {
			unset ${tryArray}
						
			return -code error "couldn't use finally block: missing closing brace, bracket or \" (try)"
		}

		# verifying the given exception for existens/definition

		foreach exception [set ${tryArray}(catch:exceptions)] {
			 if { ![${this}::Exists $exception] } {
				unset ${tryArray}
							
				return -code error "couldn't use exception \"$exception\": exception not found (try)"
			 }
		}

		# executing the try block

		global TRYCommonResult  TRYCommonErrorName  TRYCommonErrorMsg  TRYCommonErrorInfo  TRYCommonErrorCode
		global TRYCatchResult   TRYCatchErrorName   TRYCatchErrorMsg   TRYCatchErrorInfo   TRYCatchErrorCode
		global TRYFinallyResult TRYFinallyErrorName TRYFinallyErrorMsg TRYFinallyErrorInfo TRYFinallyErrorCode

		global errorInfo errorCode

		set errorInfo	""
		set errorCode	""

		if { [catch { uplevel [set ${tryArray}(common:uplevel)] [set ${tryArray}(common:block)] } TRYCommonResult] } {
			set TRYCommonErrorName	[lindex [${this}::Lastraised] 0]
			set TRYCommonErrorMsg	$TRYCommonResult
			set TRYCommonErrorInfo	$errorInfo
			set TRYCommonErrorCode	$errorCode
			set TRYCommonResult		""

			if { [${this}::Ison] && [set ${tryArray}(common:catch)] } {
				foreach exception [set ${tryArray}(catch:exceptions)] {
					if { [${this}::Israised $exception] } {
						# executing the catch block, because of the matching exception

						if { [catch { uplevel [set ${tryArray}(common:uplevel)] [set ${tryArray}(catch:$exception)] } TRYCatchResult] } {
							set TRYCatchErrorName	[lindex [${this}::Lastraised] 0]
							set TRYCatchErrorMsg	$TRYCatchResult
							set TRYCatchErrorInfo	$errorInfo
							set TRYCatchErrorCode	$errorCode
							set TRYCatchResult		""

							unset ${tryArray}

							error $TRYCatchErrorMsg $TRYCatchErrorInfo $TRYCatchErrorCode
						} else {
							set TRYCatchErrorName	""
							set TRYCatchErrorMsg	""
							set TRYCatchErrorInfo	""
							set TRYCatchErrorCode	""

							unset ${tryArray}
							
							return $TRYCatchResult
						}
					}
				}
			} 
			
			if { [set ${tryArray}(common:catch)] && ![set ${tryArray}(common:finally)] } {
				error $TRYCommonErrorMsg $TRYCommonErrorInfo $TRYCommonErrorCode
			} elseif { [set ${tryArray}(common:finally)] } {
				# executing the finally block when no catch block is defined

				# executing the finally block

				if { [catch { uplevel [set ${tryArray}(common:uplevel)] [set ${tryArray}(finally:block)] } TRYFinallyResult] } {
					set TRYFinallyErrorName	[lindex [${this}::Lastraised] 0]
					set TRYFinallyErrorMsg	$TRYFinallyResult
					set TRYFinallyErrorInfo	$errorInfo
					set TRYFinallyErrorCode	$errorCode
					set TRYFinallyResult	""

					unset ${tryArray}
							
					error $TRYFinallyErrorMsg $TRYFinallyErrorInfo $TRYFinallyErrorCode
				} else {
					set TRYFinallyErrorName	""
					set TRYFinallyErrorMsg	""
					set TRYFinallyErrorInfo	""
					set TRYFinallyErrorCode	""

					unset ${tryArray}
							
					return $TRYFinallyResult
				}
			}

			unset ${tryArray}
					
			return $TRYCommonErrorMsg
		} else {
			set TRYCommonErrorName	""
			set TRYCommonErrorMsg	""
			set TRYCommonErrorInfo	""
			set TRYCommonErrorCode	""

			unset ${tryArray}
							
			return $TRYCommonResult
		}
	} else {
		return -code error "wrong # args: should be \"try ?option? {..} ?catch exception {..} ...? ?finally {..}?\""
	}
}

$packageName

namespace inscope :: namespace import ::${packageName}::try ::${packageName}::exceptions

